* {
	padding: 0px;
	margin: 0px;
}

body {
	font-family: Helvetica, "Trebuchet MS", Arial, Helvetica, sans-serif;
	font-size: small;
}




/* Prevent hyperlinks changing color */
a ,
a:visited {
    text-decoration: none;  /*Remove underline */
	color: #036DA7;
}





/* Wrap element contains everything */
#wrap {
	margin: 20px; 
    background-color: #fff;
	border: 1px solid #aaa;
}


/* Header */
#header {
	height: 100px;
	background: #424F6F url(header.jpg);
}

#header h1 {
	color: #fff;
	font-size: 1.7em;
	padding: 15px 0 15px 10px;
	font-weight: 100;
}

#header h2 {
	color: #eee;
	font-size: 190%;
	padding: 0 0 0 10px;
	font-weight: 100;
    font-variant:small-caps;
    text-align:center;
}


/* Sidebar Formatting */
#sidebar  {
	width: 200px;
	float: left;
	background: #FFF;
	color: #666;
}

#sidebar h2 {
    padding-left: 2px;
	font-size: .9em; 
	background: #424F6F;
	height: 16px;
	color: #fff;
}

.sidebarblock ul {
	margin-left: 10px;
}
.sidebarblock li {
	list-style-type: none;
    padding: 5px;
}

#sidebar .sidebarblock {
	background: #f7f7f7;
	margin-bottom: 10px;
	padding: 5px;
	color: #666;
}



#mainpane {
    padding:5px;
    margin:5px;
	border: 2px solid #aaaaaa;
}







#content {
	background: #f7f7f7;
	margin-left: 220px;
	text-align: justify;
	color: #333;
	border: 2px solid #aaaaaa;
}

#content h2 {
	color: #555;
	font-size: 1.4em;
}



table.details  {
	white-space: nowrap;
	padding: 1px;
	text-align: left;
    margin: 2px;
}
table.details tr,
table.details td,
table.details th,
{
	padding: 0px;
}






/* Front Page Menu */
.menu {
    
margin-top: 10px;
margin-bottom: 10px;
border: 2px solid #aaaaaa;
width: 650px;
margin:0 auto;
margin-bottom: 20px;
margin-top: 20px;
}

.menu ul{
    height: 30px;
}


.menu ul li :hover {
    background-color:#ffa90c;
}
.menu ul li {
    background-color:#ffbb3e;
    float: left;
    width: 150px;
    list-style-type: none;
    text-align:center;
    padding: 3px;
}






.simulationlist {
    /*float:left;*/
    margin:10px;
}

.simulationlist td {
    margin:5px;
    padding:5px;
}

.simulationlist th {
    text-align:center;
}




.section {
	margin-left: 10px;
	margin-right: 10px;
	margin-top: 10px;
	/*margin-bottom: 10px;*/
        margin-bottom: 1px;
	padding: 4px;/*color: #555; */
}

ul.simple {
	margin-left: 2em;
	font-size: 100%;
}

ul.submissiondetailssidebar {
	font-size: 75%;

}


span.left {
position: absolute;
left: 0;
}
span.right {
position: absolute;
right: 0;
}
div.line {
position: relative;
text-align: center;
width: 100%;
}


.highlight .k { color: #AA22FF; font-weight: bold }
.highlight .s { color: #BB4444 }





/*
.SimulationNotQueued td{
	border-top: 3px solid red;
	border-bottom: 3px solid red;
    margin: 1px;
    padding: 1px;
}
*/


.filecontents,
.codeblock {
/*background-color: lightgrey;*/
background-color: #E0E0E0;
border: 1px solid black;
padding: 5px;
margin: 5px;
}

.filecontents{
width: 1000px;
}





.SummaryFileGroup
{
border: 1px solid black;
padding: 5px;
margin: 5px;

}

.SummaryFile
{
border: 1px solid black;
padding: 5px;
margin: 5px;
margin-bottom: 25px;
background-color:linen;

}
.SummaryFileDescription
{
border: 1px solid black;
padding: 5px;
margin: 5px;
color: chocolate;
font-family:"monospace";
font-weight: bold;
background-color:lavender;

}



.interactive a:hover{
background-color:yellow;
cursor:pointer;

}
